@@ -61,19 +61,81 @@ class CompileTest(TestBase, AssertsCompiledSQL):
         self.assert_compile(i, "INSERT INTO mytable (name) VALUES (%(name)s) RETURNING length(mytable.name)", dialect=dialect)
 
     def test_extract(self):
-        t = table('t', column('col1'))
 
-        for field in 'year', 'month', 'day':
-            self.assert_compile(
-                select([extract(field, t.c.col1)]),
-                "SELECT EXTRACT(%s FROM t.col1 :: timestamp) AS anon_1 "
-                "FROM t" % field)
+        t = table('t', column('col1', DateTime), column('col2', Date), column('col3', Time),
+                    column('col4', postgres.PGDateTime), column('col5', postgres.PGDate),
+                    column('col6', postgres.PGTime), column('col7', postgres.PGInterval)
+        )
 
-        for field in 'year', 'month', 'day':
-            self.assert_compile(
-                select([extract(field, func.timestamp() - datetime.timedelta(days =5))]),
-                "SELECT EXTRACT(%s FROM (timestamp() - %%(timestamp_1)s) :: timestamp) AS anon_1"
-                 % field)
+        for field in 'year', 'month', 'day', 'epoch', 'hour':
+            for expr, compiled_expr in [
+                ( t.c.col1, "t.col1 :: timestamp" ),
+                ( t.c.col2, "t.col2 :: date" ),
+                ( t.c.col3, "t.col3 :: time" ),
+                (func.current_timestamp() - datetime.timedelta(days=5),
+                    "(CURRENT_TIMESTAMP - %(current_timestamp_1)s) :: timestamp"
+                ),
+                (func.current_timestamp() + func.current_timestamp(), 
+                    "CURRENT_TIMESTAMP + CURRENT_TIMESTAMP" # invalid, no cast.
+                ),
+                (text("foo.date + foo.time"), 
+                    "foo.date + foo.time" # plain text.  no cast.
+                ),
+                (func.current_timestamp() + datetime.timedelta(days=5), 
+                    "(CURRENT_TIMESTAMP + %(current_timestamp_1)s) :: timestamp"
+                ),
+                (t.c.col2 + t.c.col3,
+                    "(t.col2 + t.col3) :: timestamp"
+                ),
+                (t.c.col5 + t.c.col6,
+                    "(t.col5 + t.col6) :: timestamp"
+                ),
+                # addition is commutative
+                (t.c.col2 + datetime.timedelta(days=5),
+                    "(t.col2 + %(col2_1)s) :: timestamp"
+                ),
+                (datetime.timedelta(days=5) + t.c.col2,
+                    "(%(col2_1)s + t.col2) :: timestamp"
+                ),
+                (t.c.col4 + t.c.col7,
+                    "(t.col4 + t.col7) :: timestamp"
+                ),
+                # subtraction is not
+                (t.c.col1 - datetime.timedelta(seconds=30),
+                    "(t.col1 - %(col1_1)s) :: timestamp"
+                ),
+                (datetime.timedelta(seconds=30) - t.c.col1,
+                    "%(col1_1)s - t.col1" # invalid - no cast.
+                ),
+                (func.coalesce(t.c.col1, func.current_timestamp()),
+                    "coalesce(t.col1, CURRENT_TIMESTAMP) :: timestamp"
+                ),
+                (t.c.col3 + datetime.timedelta(seconds=30),
+                    "(t.col3 + %(col3_1)s) :: time"
+                ),
+                (t.c.col6 + datetime.timedelta(seconds=30),
+                    "(t.col6 + %(col6_1)s) :: time"
+                ),
+                (func.current_timestamp() - func.coalesce(t.c.col1, func.current_timestamp()),
+                    "(CURRENT_TIMESTAMP - coalesce(t.col1, CURRENT_TIMESTAMP)) :: interval",
+                ),
+                (3 * func.foobar(type_=Interval),
+                    "(%(foobar_1)s * foobar()) :: interval"
+                ),
+                (literal(datetime.timedelta(seconds=10)) - literal(datetime.timedelta(seconds=10)),
+                    "(%(param_1)s - %(param_2)s) :: interval"
+                ),
+                (t.c.col3 + "some string", # dont crack up on entirely unsupported types
+                    "t.col3 + %(col3_1)s"
+                )
+            ]:
+                self.assert_compile(
+                    select([extract(field, expr)]).select_from(t),
+                    "SELECT EXTRACT(%s FROM %s) AS anon_1 FROM t" % (
+                        field, 
+                        compiled_expr
+                    )
+                )
 
 class ReturningTest(TestBase, AssertsExecutionResults):
     __only_on__ = 'postgres'
